﻿using SQLite;

namespace StationProbe
{
    internal class Database:IDisposable
    {
        private readonly SQLiteConnection _db;

        public Database()
        {
            _db = new SQLiteConnection("d:\\database.db");
            _db.CreateTable<Image>();
            _db.CreateTable<Exam>();
            _db.CreateTable<BatchTask>();
        }


        public void AddBatchTask(BatchTask batchTask)
        {
            _db.Insert(batchTask);
        }


        public void AddImage(Image image)
        {
            _db.Insert(image);
        }

        public void AddExam(Exam exam)
        {
            _db.Insert(exam);
        }

        public BatchTask[] GetBatchTasks()
        {
            return _db.Table<BatchTask>().ToArray();
        }

        public BatchTask? GetBatchTask(int id)
        {
            return _db.Table<BatchTask>().FirstOrDefault(x=>x.Id == id);
        }

        public void UpdateBatchTask(BatchTask batchTask)
        {
            _db.Update(batchTask);
        }

        public Exam[] GetExams(int batchTaskId, int page)
        {
            return _db.Table<Exam>().Where(x=>x.BatchTaskId == batchTaskId && x.PageIndex == page).OrderBy(x=>x.Id).ToArray();
        }

        public Image[] GetImages(int examId)
        {
            return _db.Table<Image>().Where(x => x.ExamId == examId).OrderBy(x=>x.ImageIndex).ToArray();
        }

        public Exam? GetLastExam(int batchTaskId)
        {
            return _db.Table<Exam>().Where(x => x.BatchTaskId == batchTaskId).OrderByDescending(x=>x.ExamIndex).FirstOrDefault();
        }

        public void Dispose()
        {
            _db?.Dispose();
        }
    }


}
