﻿using PuppeteerSharp;

namespace StationProbe
{
    internal class SuperImageTask : IDisposable
    {
        private readonly Database _db;
        private readonly BatchTask _batchTask;
        private readonly int _examCountInPage;
        private int _pageIndex = 1;
        private int _examIndex = 0;
        private int _examIndexInPage = 0;

        public SuperImageTask(BatchTask batchTask, int examCountInPage)
        {
            _batchTask = batchTask;
            _examCountInPage = examCountInPage;
            _db = new Database();
        }

        public async Task RunAsync(IPage page)
        {
            IPage output = page;
            var url = "http://192.168.200.85:8080";
            Logger.WriteLine($"Start login task (url:{url})...");
            try
            {
                output = await new LoginTask(url).ExecuteAsync(output,_db);
            }
            catch (Exception ex)
            {
                Logger.WriteLine($"Run LoginTask failed:{ex}");
                throw;
            }
            Logger.WriteLine("Start menu task...");
            try
            {
                output = await new MenuTask().ExecuteAsync(output, _db);
            }
            catch (Exception ex)
            {
                Logger.WriteLine($"Run MenuTask failed:{ex}");
                throw;
            }
            Logger.WriteLine($"Start filter task（start:{_batchTask.Start:yyyy-MM-dd}, end:{_batchTask.End:yyyy-MM-dd}）...");
            try
            {
                var filterTask = new FilterTask(_batchTask.Start, _batchTask.End);
                output = await filterTask.ExecuteAsync(output, _db);
                if (filterTask.PageCount != _batchTask.PageCount || filterTask.ExamCount != _batchTask.ExamCount)
                {
                    //Update the batchTask
                    _batchTask.ExamCount = filterTask.ExamCount;
                    _batchTask.PageCount = filterTask.PageCount;
                    _db.UpdateBatchTask(_batchTask);
                }
                Logger.WriteLine($"Get page count:{filterTask.PageCount}, exam count:{filterTask.ExamCount}");
            }
            catch (Exception ex)
            {
                Logger.WriteLine($"Run FilterTask failed:{ex}");
                throw;
            }

            var lastExam = _db.GetLastExam(_batchTask.Id);
            if (lastExam != null)
            {
                _pageIndex = lastExam.PageIndex;
                _examIndex = lastExam.ExamIndex + 1;
                _examIndexInPage = lastExam.ExamIndexInPage + 1;
                if (_examIndexInPage + 1 == _examCountInPage)
                {
                    _pageIndex++;
                    _examIndexInPage = 0;
                }
            }
            Logger.WriteLine($"Start from page: {_pageIndex}, exam index: {_examIndexInPage}");
            while (_pageIndex < _batchTask.PageCount)
            {
                Logger.WriteLine($"Start page task (page:{_pageIndex})");
                try
                {
                    var pageTask = new PageTask(_pageIndex);
                    output = await pageTask.ExecuteAsync(output, _db);
                    var currentExamCountinPage = _pageIndex < (_batchTask.PageCount - 1) ? _examCountInPage : (_batchTask.ExamCount - _pageIndex * _examCountInPage);
                    while (_examIndexInPage < currentExamCountinPage)
                    {
                        Logger.WriteLine($"Start exam task (exam index: {_examIndexInPage})...");
                        try
                        {
                            output = await new ExamTask(_batchTask.Id, _pageIndex, _examIndex ,_examIndexInPage).ExecuteAsync(output, _db);
                        }
                        catch (Exception ex)
                        {
                            Logger.WriteLine($"Run ExamTask failed:{ex}");
                            throw;
                        }
                    }
                    _examIndexInPage = 0;
                }
                catch (Exception ex)
                {
                    Logger.WriteLine($"Run PageTask failed:{ex}");
                    throw;
                }
                _pageIndex++;
            }
        }

        public void Dispose()
        {
            _db.Dispose();
        }
    }
}
